#!/usr/bin/env python
import asyncio
import logging
import os
import shutil
from os import path
from pathlib import Path

import click
import datayoga_core as dy
import jsonschema
from datayoga.cli_helpers import handle_critical
from datayoga_core import utils
from pkg_resources import get_distribution

from datayoga import cli_helpers

CONTEXT_SETTINGS = dict(max_content_width=120)
LOG_LEVEL_OPTION = [click.option(
    "--loglevel", '-log-level', type=click.Choice(
        ["DEBUG", "INFO", "WARN", "ERROR", "CRITICAL"],
        case_sensitive=False),
    default="INFO", show_default=True)]

logger = logging.getLogger("dy")

# create console handler with a higher log level
ch = logging.StreamHandler()
ch.setFormatter(cli_helpers.CustomFormatter())
logger.addHandler(ch)


@click.group(name="datayoga", help="DataYoga command line tool")
@click.version_option(get_distribution("datayoga").version)
def cli():
    pass


@cli.command(name="init", help="Scaffolds a new folder with sample configuration files")
@click.argument("project_name")
@cli_helpers.add_options(LOG_LEVEL_OPTION)
def init(
    project_name: str,
    loglevel: str
):
    set_logging_level(loglevel)

    try:
        if os.path.exists(project_name):
            raise ValueError(f"{project_name} is already exists")

        logger.debug(f"Creating {project_name} directory")
        shutil.copytree(utils.get_resource_path("scaffold"), project_name)

        # Explicitly set permissions for created files and directories, because in some cases this may not be correct.
        for root, dirs, files in os.walk(project_name):
            for paths, permissions in ((files, "644"), (dirs, "755")):
                for path in paths:
                    os.chmod(os.path.join(root, path), int(permissions, 8))

        print(f"Initialized {project_name} successfully")
    except Exception as e:
        handle_critical(logger, f"Error while initializing {project_name}", e)


@cli.command(name="validate", help="Validates a job in dry run mode")
@cli_helpers.add_options(LOG_LEVEL_OPTION)
def validate(
        loglevel: str
):
    set_logging_level(loglevel)


@cli.command(name="run", help="Runs a job", context_settings=CONTEXT_SETTINGS)
@click.argument("job_name")
@click.option('--dir', 'directory', help="DataYoga directory", default=".", show_default=True)
@cli_helpers.add_options(LOG_LEVEL_OPTION)
def run(
    job_name: str,
    directory: str,
    loglevel: str
):
    set_logging_level(loglevel)

    try:
        logger.info("Runner started...")
        job_file = path.join(directory, "jobs", job_name.replace(".", os.sep) + ".yaml")
        job_settings = utils.read_yaml(job_file)
        logger.debug(f"job_settings: {job_settings}")

        connections = utils.read_yaml(path.join(directory, "connections.yaml"))
        logger.debug(f"connections: {connections}")

        jsonschema.validate(instance=connections, schema=utils.read_json(
            utils.get_resource_path(os.path.join("schemas", "connections.schema.json"))))

        context = dy.Context({
            "connections": connections,
            "data_path": path.join(directory, "data"),
            "job_name": Path(job_file).stem
        })

        job = dy.compile(job_settings)

        producer = job.input
        logger.info(f"Producing from {producer.__module__}")
        job.init(context)
        asyncio.run(job.run())
    except Exception as e:
        cli_helpers.handle_critical(logger, "Error while running a job", e)


@cli.command(name="test", help="Unit test one or more job using data test definitions")
@cli_helpers.add_options(LOG_LEVEL_OPTION)
def test(
    loglevel: str
):
    set_logging_level(loglevel)


@cli.command(name="status", help="Displays status and statistics of running and completed jobs")
@cli_helpers.add_options(LOG_LEVEL_OPTION)
def status(
    loglevel: str
):
    set_logging_level(loglevel)


@cli.command(name="stop", help="Stops a job")
@cli_helpers.add_options(LOG_LEVEL_OPTION)
def stop(
    loglevel: str
):
    set_logging_level(loglevel)


@cli.command(name="start", help="Starts a job")
@cli_helpers.add_options(LOG_LEVEL_OPTION)
def start(
    loglevel: str
):
    set_logging_level(loglevel)


@cli.command(name="preview", help="Previews a job using sample data")
@cli_helpers.add_options(LOG_LEVEL_OPTION)
def trace(
    loglevel: str
):
    set_logging_level(loglevel)


def set_logging_level(loglevel: str):
    """Sets logger's log level

    Args:
        loglevel(str): log level
    """
    logger.setLevel(loglevel.upper())
    ch.setLevel(loglevel.upper())


def main():
    cli()


if __name__ == "__main__":
    cli()



import logging
import sys
import traceback
from logging import Formatter, Logger
from typing import Callable, Sequence


class CustomFormatter(Formatter):
    class Bcolors:
        HEADER = '\033[95m'
        GREY = "\x1b[38;20m"
        BLUE = '\033[94m'
        GREEN = '\033[92m'
        YELLOW = '\033[93m'
        RED = '\033[91m'
        ENDC = '\033[0m'
        BOLD = '\033[1m'
        UNDERLINE = '\033[4m'
        RESET = "\x1b[0m"

    format = "%(levelname)s - %(message)s"

    FORMATS = {
        logging.DEBUG: Bcolors.GREY + format + Bcolors.RESET,
        logging.INFO: Bcolors.GREY + format + Bcolors.RESET,
        logging.WARNING: Bcolors.YELLOW + format + Bcolors.RESET,
        logging.ERROR: Bcolors.RED + format + Bcolors.RESET,
        logging.CRITICAL: Bcolors.RED + Bcolors.BOLD + format + Bcolors.RESET
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


def add_options(options: Sequence[Callable]):
    def _add_options(func):
        for option in reversed(options):
            func = option(func)
        return func

    return _add_options


def handle_critical(logger: Logger, msg: str, e: Exception):
    logger.critical(f"{msg}:\n{e}")
    if logger and logger.isEnabledFor(logging.DEBUG):
        traceback.print_exc()

    sys.exit(1)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['datayoga']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'certifi>=2022.12.7,<2023.0.0',
 'click>=8.0.3,<9.0.0',
 'datayoga-core>=1.0.0,<2.0.0',
 'jsonschema>=4.4.0,<5.0.0']

extras_require = \
{'test': ['mock>=4.0.3,<5.0.0',
          'pytest>=7.1.2,<8.0.0',
          'pytest-describe>=2.0.1,<3.0.0',
          'pytest-mock>=3.7.0,<4.0.0',
          'pytest-timeout>=2.1.0,<3.0.0',
          'requests-mock>=1.9.3,<2.0.0']}

entry_points = \
{'console_scripts': ['datayoga = datayoga.__main__:main']}

setup_kwargs = {
    'name': 'datayoga',
    'version': '1.60.0',
    'description': 'DataYoga command line interface',
    'long_description': '# DataYoga CLI\n\n## Development\n\nTo set up environment in development mode:\n\n### Set Up Virtual Environment\n\n```bash\npython -m venv venv\nsource venv/bin/activate\n```\n\n### Upgrade `pip` to Latest Version\n\n> [Pip](https://pypi.org/project/pip) version 22 and up is needed for editable install.\n\n```bash\npython -m pip install --upgrade pip\n```\n\n### Install Dependencies\n\n```bash\ncd core\npython -m pip install -e .\n```\n\n## Run CLI in Development Mode\n\n```bash\npython ./cli/src/datayoga/__main__.py\n```\n\n## Running tests\n\n### Core\n\n```\ncd core\npip install .\npip install .[test]\nPYTHONPATH=src python -m pytest -s --log-cli-level=DEBUG\n```\n',
    'author': 'DataYoga',
    'author_email': 'admin@datayoga.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
